using System;
using System.Drawing;
using MonoTouch.Foundation;
using MonoTouch.UIKit;
using Infragistics;
using System.Collections.Generic;

namespace CustomCell
{
	public partial class CustomCellViewController : UIViewController
	{

		IGGridViewDataSourceHelper ds;
		IGGridView grid;
		List<Data> data;
		UITapGestureRecognizer _tapGesture;

		static bool UserInterfaceIdiomIsPhone {
			get { return UIDevice.CurrentDevice.UserInterfaceIdiom == UIUserInterfaceIdiom.Phone; }
		}

		public CustomCellViewController ()
			: base ()
		{
		}
		public override void ViewDidLoad ()
		{
			base.ViewDidLoad ();

			this.View.BackgroundColor = UIColor.White;

			data = Data.generateData (150);

			grid = new IGGridView (new RectangleF (0, 0, this.View.Bounds.Size.Width , this.View.Bounds.Size.Height),
			                       IGGridViewStyle.IGGridViewStyleDefault);

			grid.AutoresizingMask = UIViewAutoresizing.FlexibleWidth | UIViewAutoresizing.FlexibleHeight;

			this.View.Add (grid);


			ds = new IGGridViewDataSourceHelper();
			ds.ColumnDefinitions.AddObjects(new NSObject[]{new IGGridViewColumnDefinition("Identifier"), new DateColumn("Date"), new TextColumn("Name")});

			ds.Data = data.ToArray ();

			grid.DataSource = ds;

			UIBarButtonItem item = new UIBarButtonItem ("Add", UIBarButtonItemStyle.Plain, new EventHandler (this.AddRow));

			this.SetToolbarItems (new UIBarButtonItem[] { item }, false);

			_tapGesture = new UITapGestureRecognizer( new Action<UITapGestureRecognizer>(TapGestureRecognizer));


		}

		public void AddRow(object sender, EventArgs e)
		{
			UIBarButtonItem item = (UIBarButtonItem)sender;

			if (item.Title == "Add") 
			{
				Data d = new Data ();
				data.Add (d);

				ds.Data = data.ToArray();

				grid.UpdateData ();
				grid.ScrollToCell (new IGCellPath (data.Count - 1, 0, 0), IGGridViewScrollPosition.IGGridViewScrollPositionBottomLeft, true);

				grid.AddGestureRecognizer (_tapGesture);

				item.Title = "Done";
			} 
			else 
			{
				item.Title = "Add";

				grid.RemoveGestureRecognizer (_tapGesture);
			}


		}


		public void TapGestureRecognizer (UITapGestureRecognizer gesture)
		{
			IGCellPath path = grid.GetCellPathAtPoint (gesture.LocationInView (grid));

			if (path.RowIndex == data.Count - 1) 
			{
				IGGridViewCellBase cell = (IGGridViewCellBase)grid.ResolveCellForPath (path);

				if (cell is DateCell) 
				{

				} 
				else if (cell is TextCell) 
				{
					((TextCell)cell).StartEditing ();
				}
			}

		}
	}

	public class BaseColumn : IGGridViewColumnDefinition
	{
		string _fieldKey;
		public override string FieldKey {
			get {
				return _fieldKey;
			}
		}

		public BaseColumn(string key):base()
		{
			_fieldKey = key;
		}
	}

	public class DateColumn : BaseColumn
	{
		public DateColumn(string key):base(key)
		{

		}

		public override IGGridViewCell CreateCell (IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource)
		{
			DateCell cell = (DateCell)gridView.DequeueReusableCell ("DateCell");

			if (cell == null)
				cell = new DateCell ("DateCell");

			DateTime dt = (NSDate)dataSource.ResolveDataValueForCell (path);

			cell.TextLabel.Text = dt.ToShortDateString ();

		


			return cell;
		}
	}

	public class TextColumn : BaseColumn
	{
		public TextColumn(string key):base(key)
		{

		}

		public override IGGridViewCell CreateCell (IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource)
		{
			TextCell cell = (TextCell)gridView.DequeueReusableCell ("TextCell");

			if (cell == null)
				cell = new TextCell ("TextCell");


			cell.TextLabel.Text = dataSource.ResolveDataValueForCell (path).ToString ();


			return cell;
		}
	}

	public class DateCell :IGGridViewCell
	{
		public DateCell(string identifier):base(identifier)
		{

		}

	}

	public class TextCell : IGGridViewCell
	{
		UITextField _field;

		public TextCell(string identifier):base(identifier)
		{
			_field = new UITextField ();
		}

		public void StartEditing()
		{
			this.AddSubview (_field);
			this.SendSubviewToBack (_field);

			_field.BecomeFirstResponder ();

			UIView.Transition(this.TextLabel, _field, .25f, UIViewAnimationOptions.TransitionFlipFromTop, null);
		}

		public override void CellDetached ()
		{
			_field.RemoveFromSuperview ();
		}

		public override void SetupSize (SizeF size)
		{
			base.SetupSize (size);
			_field.Frame = this.TextLabel.Frame;
		}

	}
}

